/*
 * SyncVector.h
 *
 *  Created on: Feb 20, 2017
 *      Author: vagner
 */

#pragma once

#include <chrono>
#include <mutex>
#include <vector>

#ifdef __KOS__
#include <time.h>
#include <semaphore.h>
#endif

namespace vCommon
{
	//
	//	Базовое сообщение.
	//
	class svMessage
	{
	public:
		int OpCode;		//	код операции
		//
		//	ctor
		//
		svMessage ( int OpCode )
			: OpCode ( OpCode )
		{
		}
		//
		//	dtor
		//
		virtual ~svMessage ( )
		{
		}
	};
	//
	//	Синхронизированный вектор.
	//
	class SyncVector
	{
	public:
		//
		//	ctor
		//
		SyncVector ( );
		//
		//	dtor
		//
		~SyncVector ( );
		//
		//	Очистить очередь.
		//
		void Clear ( );
		//
		//	Количество сообщений.
		//
		int Count ( );
		//
		//	Получить первое сообщение очереди.
		//	При необходимости (msec > 0): дождаться появления сообщения.
		//
		svMessage* PopWait ( int msec, bool IsRemove );
		//
		//	Получить первое сообщение очереди.
		//
		svMessage* Pop ( );
		//
		//	Записать в хвост очереди.
		//
		size_t Push ( svMessage* msg );

	private:
#ifndef __KOS__
		std::timed_mutex mxWait;		//	семафор ожидания
#else
		struct timespec ts;        		//  timespec for semaphore
		sem_t sem;              		//  семафор ожидания
#endif
		std::mutex mxLock;				//	семафор блокировки
		std::vector<svMessage*> Items;	//	очередь
		//
		//	Прочитать первое сообщение очереди и удалить его.
		//	Внимание: выполняется без блокировки очереди!!!
		//
		svMessage* get_remove ( bool IsRemove );
	};

} /* namespace vCommon */
