#pragma once

#include <map>
#include <string>

#include <jsoncpp/json/json.h>

#include "vdef.h"

namespace vCommon
{
	//-----------------------------------------------------------------------------
	//	Типы значений
	enum class ArgValueType
	{
		avByte
		, avBool
		, avU16
		, avU32
		, avU64
		, avFloat
		, avString
	};

	//-----------------------------------------------------------------------------
	//	Значение аргумента.
	union ArgValue
	{
		byte vb;
		bool vbool;
		u16 v16;
		u32 v32;
		u64 v64;
		float vf;
		std::string vstr;

		ArgValue ( byte v )
						: vb ( v )
		{
		}
		ArgValue ( bool v )
						: vbool ( v )
		{
		}
		ArgValue ( u16 v )
						: v16 ( v )
		{
		}
		ArgValue ( u32 v )
						: v32 ( v )
		{
		}
		ArgValue ( u64 v )
						: v64 ( v )
		{
		}
		ArgValue ( float v )
						: vf ( v )
		{
		}
		ArgValue ( std::string v )
						: vstr ( v )
		{
		}

		~ArgValue ( )
		{
		}
	};

	//-----------------------------------------------------------------------------
	//	Аргумент
	class vArg
	{
	public:

		//	имя параметра
		std::string Name;

		//	числовое значение
		ArgValue value;

		//	тип значения
		ArgValueType type;

		//	ctor
		vArg ( const std::string& name, byte v );
		vArg ( const std::string& name, bool v );
		vArg ( const std::string& name, u16 v );
		vArg ( const std::string& name, u32 v );
		vArg ( const std::string& name, u64 v );
		vArg ( const std::string& name, float v );
		vArg ( const std::string& name, const std::string& v );

		//	ToString
		inline std::string ToString ( )
		{
			switch ( type )
			{
				case ArgValueType::avBool:
					return std::to_string ( value.vbool );
				case ArgValueType::avByte:
					return std::to_string ( value.vb );
				case ArgValueType::avFloat:
					return std::to_string ( value.vf );
				case ArgValueType::avString:
					return value.vstr;
				case ArgValueType::avU16:
					return std::to_string ( value.v16 );
				case ArgValueType::avU32:
					return std::to_string ( value.v32 );
				case ArgValueType::avU64:
					return std::to_string ( value.v64 );
			}

			return std::string ( );
		}
	};

	//-----------------------------------------------------------------------------
	//	Список аргументов
	class vArgList
	{
	public:

		//	список аргументов
		std::map <std::string, vArg*> Items;

		//	ctor
		vArgList ( )
						: Items ( )
		{
		}

		//	dtor
		~vArgList ( );

		//	добавить аргумент
		void add ( const std::string& aname, byte v );
		void add ( const std::string& aname, bool v );
		void add ( const std::string& aname, u16 v );
		void add ( const std::string& aname, u32 v );
		void add ( const std::string& aname, u64 v );
		void add ( const std::string& aname, float v );
		void add ( const std::string& aname, std::string& v );
		void add ( const std::string& aname, Json::Value& v );

		//	получить аргумент
		vArg* get ( const std::string& aname );
		byte getByte ( const std::string& aname, byte def );
		bool getBool ( const std::string& aname, bool def );
		u16 getU16 ( const std::string& aname, u16 def );
		i16 getI16 ( const std::string& aname, i16 def );
		u32 getU32 ( const std::string& aname, u32 def );
		i32 getI32 ( const std::string& aname, i32 def );
		u64 getU64 ( const std::string& aname, u64 def );
		i64 getI64 ( const std::string& aname, i64 def );
		float getFloat ( const std::string& aname, float def );
		std::string getString ( const std::string& aname, std::string def );
	};

} /* namespace vCommon */
