/*
 * mqClient.h
 *
 *  Created on: Feb 22, 2017
 *      Author: vagner
 */

#pragma once

#include <mutex>
#include <string>
#include <vector>

#include <mosquitto.h>
#include <SyncVector.h>
#include "mqMsg.h"

namespace vMQ
{
	//
	//	Элемент подписки
	//
	class subsItem
	{
	public:
		std::string Topic;	//	топик
		int qos;			//	качество обслуживания
		//
		//	ctor
		//
		subsItem ( const std::string& Topic, int qos )
			: Topic ( Topic ), qos ( qos )
		{
		}
	};
	//
	//	Клиент mosquitto
	//
	class mqClient
	{
	private:

		//	клиент MQTT
		mosquitto* mqtt;

		//	флаг очистки сессии при установлении соединения
		bool CleanSession;

		//	ClientId
		std::string ClientId;

		//	очередь входящих
		vCommon::SyncVector* in;

		//	очередь исходящих
		//	Внутренняя очередь клиента
		vCommon::SyncVector out;

		//	флаг уничтожения объекта
		bool IsDisposed;

		//	семафор блокировки обработчиков событий
		std::mutex mxCallBack;

		//	семафор доступа к флагам
		std::mutex mxFlag;

		//	семафор отправки
		std::mutex mxSend;

		//	MQTT broker URL
		std::string Url;

		//	MQTT broker Port
		int Port;

		//	установить значение bool
		void setBool ( bool& flag, bool value );
		//	Отправить первое сообщение из очереди исходящих.
		void send ( );

		//
		//	<Обработчики событий>
		//
		//	on_connect
		static void on_connect ( mosquitto* mqtt, void* context, int rc );
		//	on_disconnect
		static void on_disconnect ( mosquitto* mqtt, void* context, int rc );
		//	on_message
		static void on_message ( mosquitto* mqtt, void* context, const mosquitto_message* msg );
		//	on_publish
		static void on_publish ( mosquitto* mqtt, void* context, int mid );
		//	on_subscribe
		static void on_subscribe ( mosquitto* mqtt, void* context, int mid, int count, const int* qosList );
		//	on_unsubscribe
		static void on_unsubscribe ( mosquitto* mqtt, void* context, int mid );
        //  on_log
        static void on_log ( mosquitto* mqtt, void* context, int level, const char * str );

	public:
		static const int mqEvent_Connected = 0;
		static const int mqEvent_Message = 1;
		static const int mqEvent_Disconnected = 2;
		//	флаг наличия соединения
		bool IsConnected;
		int keepAlive;

		//	ctor
		mqClient ( bool CleanSession, vCommon::SyncVector* in, std::string ClientId, int keepAlive = 60 );
		//	dtor
		~mqClient ( );
		//	connect
		bool connect ( const std::string& Url, int Port );
		//	disconnect
		void disconnect ( );
		//	publish
        bool publish ( const std::string& Topic, const std::string& payload, int qos = 2, bool retain = false );
        bool publish ( mqMsg* msg );
		//	subscribe
		bool subscribe ( const std::vector<subsItem>& SubsList );
		//	unsubscribe
		bool unsubscribe ( );
		//	получить значение bool
		bool getBool ( const bool& flag );
	};

} /* namespace vMQ */
