#pragma once

#include <string>

#include <vCommon.h>

namespace vsock
{
	//	Синхронная работа с сокетом.
	class vsock
	{
	public:
		//	конструктор
		//	name - логическое имя
		//	rOut - таймаут приема (msec)
		vsock ( const std::string& name, int rOut, int sOut );

		//	деструктор
		~vsock ( );

		//	открыть соединение
		bool Open ( const std::string& Host, const int Port );

		//	закрыть соединение
		void Close ( );

		//	пощупать прием
		//	возвращает:
		//	<0	ошибка
		//	 0	разрыв связи
		//	>0	размер принятого
		int Peek ( void* buf, int bSize );

		//	принять
		//	возвращает:
		//	<0	ошибка
		//	 0	разрыв связи
		//	>0	размер принятого
		int Recv ( void* buf, int bSize );

		//	принять указанный размер
		bool RecvAll ( void* buf, int bSize );

		//	отправить
		//	возвращает:
		//	<0	ошибка
		//	>0	размер переданного
		int Send ( void* buf, int bSize );

		//	отправить указанный размер
		bool SendAll ( void* buf, int bSize );

		//	наличие соединения
		inline bool IsConnected ( )
		{
			return _IsConnected;
		}

		//	валидность сокета
		inline bool IsValidSocket ( )
		{
			return sockfd > 0;
		}

	private:
		//	логическое имя
		std::string name;

		//	дескриптор файла
		int sockfd;

		//	соединение установлено
		bool _IsConnected;

		//	таймаут приема
		int rOut;

		//	таймаут передачи
		int sOut;

		//	опрос
		bool poll ( short int flag, int t_out );

		//	журнал
		void log ( LMT mt, const std::string& msg );
	};

} /* namespace vsock */
